#include <stdio.h>
#include <parallel.h>

#define DEBUG_read_matrix
void read_order(char *filename, short **IBUSO, int *NBUS)
{
   FILE *Datafile;
   int   i;
   int   j;
   int   NODEA;

   if ((Datafile = fopen(filename, "r")) != NULL)
   {
      fscanf(Datafile, "NBUS = %d%*c", NBUS);
#ifdef DEBUG_read_order
      print("NBUS = %d\n", *NBUS);
#endif
      if (*IBUSO == NULL)
      {
         if ((*IBUSO = (short *) malloc(*NBUS * sizeof(short))) == NULL)
         {
            print("read_order(): Out of memory!\n");
            exit(1);
         }
      }

      for (i = 0; i < *NBUS; i++)
      {
         fscanf(Datafile, "Order = [%d] -> (%d)%*c", &j, &NODEA);
#ifdef DEBUG_read_order
         print("Order = [%d] -> (%d)\n", j, NODEA);
#endif
         (*IBUSO)[j] = (short) NODEA;
      }
   }
   else
   {
      printf("read_order(): Cann't open order data file\n");
      exit(1);
   }
}

void read_matrix(char *filename, matrix_host_bin *Matrix)
{
   FILE   *Datafile;
   int     i;
   int     j;
   int     I;
   int     J;
   complex Value;

   print("reading file %s\n", filename);
   if ((Datafile = fopen(filename, "r")) != NULL)
   {
      while (fscanf(Datafile, "A[%d][%d] (%d, %d) = (%f, %f)%*c",
              &i, &j, &I, &J, &(Value.real), &(Value.imag)) > 0)
      {
#ifdef DEBUG_read_matrix
         print("A[%d][%d] (%d, %d) = (%g, %g)\n", i, j, I, J,
               Value.real, Value.imag);
#endif
         if (I == J)
         {
            Save_matrix_data_diagonal(I, Value, Matrix);
         }
         else
         {
            Save_matrix_data_off_diagonal(I, J, Value, Matrix);
         }
      }
   }
   else
   {
      print("read_matrix(): Cann't open matrix data file\n");
      exit(1);
   }
}

#define DEBUG_read_vector

void read_vector(char *filename, vector_host_bin *Vector)
{
   FILE      *Datafile;

   int        M               = Vector -> M;

   int        P               = Vector -> P;

   void     (*Row_Distribution)();
   void      *Row_Extra       = Vector -> Row_Extra;

   element  **b               = Vector -> b;
   int       *b_element_Count = Vector -> b_element_Count;

   element   *Pool            = Vector -> Pool;

   int        I;
   int        ihat;
   int        Ihat;
   int        Offset;
   index      p;
   index      i;
   index      k;

   element   *new_element;
   complex    Value;

   print("read_vector(): entry");
   Row_Distribution = Vector -> Row_Distribution;

   Offset = 1;

   if ((Datafile = fopen(filename, "r")) != NULL)
   {
      print("read_vector(): checkpoint 1");
      while (fscanf(Datafile, "V[%d] (%d) = (%f, %f)%*c",
              &ihat, &Ihat, &(Value.real), &(Value.imag)) > 0)
      {
         print("V[%d] (%d) = (%g, %g)\n", i, Ihat, Value.real, Value.imag);

         I = Ihat << 1;

         (*Row_Distribution)(I, &p, &i, P, M, Row_Extra);
         new_element = get_new_complex_element(&Pool, i, 0, Value.real, 0.0);
         new_element -> Next = b[p];
         b[p]= new_element;
         b_element_Count[p]++;

#ifdef DEBUG_read_vector
         print("(p, i) = (%hd, %hd) Real b[%d] = %lf\n", p, i, I, Value.real);
#endif
         (*Row_Distribution)(Offset + I, &p, &i, P, M, Row_Extra);
         new_element = get_new_complex_element(&Pool, i, 0, Value.imag, 0.0);
         new_element -> Next = b[p];
         b[p]= new_element;
         b_element_Count[p]++;

#ifdef DEBUG_read_vector
         print("(p, i) = (%hd, %hd) Imag b[%d] = %lf\n", p, i, Offset + I,
               Value.imag);
#endif
      }
   }
   else
   {
      print("read_vector(): Cann't open vector data file\n");
      exit(1);
   }

   Vector -> Pool = Pool;
}

void write_vector(char *filename, vector_host_bin *Vector)
{
   FILE *Datafile;
   element   *Pointer;

   if ((Datafile = fopen(filename, "w")) != NULL)
   {
      Pointer = Vector -> Col_major_x;

      while (Pointer != NULL)
      {
         fprintf(Datafile, " V (%d) = (%g, %g)\n", Pointer -> j,
                Pointer -> Real, Pointer -> Imag);
         Pointer = Pointer -> Next;
      }
   }
   else
   {
      print("read_vector(): Cann't open vector data file\n");
      exit(1);
   }
}

